package ink.wgink.module.department.service;

import ink.wgink.interfaces.department.IDepartmentBaseService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.bos.DepartmentBO;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.department.DepartmentDTO;
import ink.wgink.pojo.dtos.department.DepartmentSimpleDTO;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.pojo.result.UploadExcelResultDTO;
import ink.wgink.module.department.pojo.vos.DepartmentVO;
import ink.wgink.module.department.pojo.vos.MergeDepartmentInfoVO;
import ink.wgink.module.department.pojo.vos.MergeNewDepartmentInfoVO;
import ink.wgink.module.department.pojo.vos.SplitDepartmentVO;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IDepartmentService
 * @Description: 组织部门
 * @Author: wanggeng
 * @Date: 2021/1/27 11:27 上午
 * @Version: 1.0
 */
public interface IDepartmentService extends IDepartmentBaseService {

    /**
     * 新增组织部门
     *
     * @param departmentVO
     * @return
     */
    void save(DepartmentVO departmentVO);

    /**
     * 保存部门返回ID
     *
     * @param departmentVO
     * @return
     */
    String saveReturnId(DepartmentVO departmentVO);

    /**
     * 删除组织部门
     *
     * @param ids
     * @return
     */
    void remove(List<String> ids);

    /**
     * 修改组织部门
     *
     * @param departmentId
     * @param departmentVO
     * @return
     */
    void update(String departmentId, DepartmentVO departmentVO);

    /**
     * 更新合并到新增组织
     *
     * @param mergeNewDepartmentInfoVO
     * @return
     * @throws Exception
     */
    void updateMergeNew(MergeNewDepartmentInfoVO mergeNewDepartmentInfoVO);

    /**
     * 更新合并到组织
     *
     * @param departmentId
     * @param mergeDepartmentInfoVO
     * @return
     */
    void updateMerge(String departmentId, MergeDepartmentInfoVO mergeDepartmentInfoVO);

    /**
     * 更新分割部门
     *
     * @param departmentId
     * @param splitDepartmentVO
     * @return
     * @throws Exception
     */
    void updateSplit(String departmentId, SplitDepartmentVO splitDepartmentVO);

    /**
     * 重置组织部门编码
     *
     * @return
     */
    void resetCode();

    /**
     * 部门列表
     *
     * @param params
     * @return
     */
    List<DepartmentDTO> list(Map<String, Object> params);

    /**
     * 通过ID获取组织部门详情
     *
     * @param departmentId
     * @return
     */
    DepartmentDTO get(String departmentId);

    /**
     * easyui组织部门
     *
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPage(ListPage page);

    /**
     * easyui组织部门列表
     *
     * @param departmentParentId
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPageByParentId(String departmentParentId, ListPage page);

    /**
     * 组织部门列表，递归获取全部内容
     *
     * @param params
     * @return
     */
    List<DepartmentDTO> listAll(Map<String, Object> params);

    /**
     * zTree组织部门
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(Map<String, Object> params);

    /**
     * zTree组织部门
     *
     * @param departmentParentId
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(String departmentParentId, Map<String, Object> params);

    /**
     * 部门ID列表
     *
     * @param params
     * @return
     */
    List<String> listId(Map<String, Object> params);

    /**
     * 获取基础部门ID列表
     *
     * @param departmentBOs
     * @return
     */
    List<String> listId(List<DepartmentBO> departmentBOs);

    /**
     * 获取上级部门
     *
     * @param departmentBO1
     * @param departmentBO2
     * @return
     */
    DepartmentBO getHigherLevel(DepartmentBO departmentBO1, DepartmentBO departmentBO2);

    /**
     * 保存部门Excel
     *
     * @param excel
     * @return
     */
    UploadExcelResultDTO uploadExcel(MultipartFile excel) throws IOException;

    /**
     * 获得同一级部门数量（按部门名称）
     *
     * @param params
     * @return
     */
    Integer count(Map<String, Object> params);

    /**
     * 通过ID获取组织部门详情（简单格式）
     *
     * @param departmentId
     * @return
     */
    DepartmentSimpleDTO getSimple(String departmentId);

    /**
     * 部门列表
     *
     * @param params
     * @return
     */
    List<DepartmentSimpleDTO> listSimple(Map<String, Object> params);

    /**
     * 通过上级ID获取组织部门全部列表（简单格式）
     *
     * @param departmentParentId
     * @return
     */
    List<DepartmentSimpleDTO> listSimpleByParentId(String departmentParentId);

    /**
     * 通过上级ID和添加获取组织部门全部增量列表（简单格式）
     *
     * @param departmentParentId
     * @param addDate
     * @return
     */
    List<DepartmentSimpleDTO> listSimpleByParentIdAndAddDate(String departmentParentId, String addDate);

    /**
     * 通过上级ID和更新获取组织部门全部新增和修改的列表（简单格式）
     *
     * @param departmentParentId
     * @param updateDate
     * @return
     */
    List<DepartmentSimpleDTO> listSimpleByParentIdAndUpdateDate(String departmentParentId, String updateDate);

}
