package ink.wgink.module.department.service.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.pojo.dtos.department.DepartmentSimpleDTO;
import ink.wgink.module.department.dao.IDepartmentUserDao;
import ink.wgink.module.department.service.IDepartmentAdjustmentService;
import ink.wgink.module.department.service.IDepartmentService;
import ink.wgink.module.department.service.IDepartmentUserService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: DepartmentUserServiceImpl
 * @Description: 组织部门用户
 * @Author: wanggeng
 * @Date: 2021/1/28 4:21 下午
 * @Version: 1.0
 */
@Service
public class DepartmentUserServiceImpl extends DefaultBaseService implements IDepartmentUserService {

    @Autowired
    private IDepartmentUserDao departmentUserDao;
    @Autowired
    private IDepartmentService departmentService;
    @Autowired
    private IDepartmentAdjustmentService departmentAdjustmentService;

    @Override
    public void saveUserIdByDepartmentId(String departmentId, String departmentName, List<String> departmentUserIds) {
        Map<String, Object> params = getHashMap(2);
        for (String departmentUserId : departmentUserIds) {
            params.put("userId", departmentUserId);
            params.put("departmentId", departmentId);
            departmentUserDao.save(params);
        }
        departmentAdjustmentService.addUserDepartmentAdjustment(departmentUserIds, departmentId, departmentName, USER_DEPARTMENT_TYPE_JOIN);
    }

    @Override
    public void deleteUserByDepartmentIdAndUserIds(String departmentId, String departmentName, List<String> userIds) {
        String adjustmentDepartmentName = departmentName;
        if (StringUtils.isBlank(departmentName)) {
            DepartmentSimpleDTO departmentSimpleDTO = departmentService.getSimple(departmentId);
            if (departmentSimpleDTO == null) {
                throw new SearchException("部门不存在");
            }
            adjustmentDepartmentName = departmentSimpleDTO.getDepartmentName();
        }
        if (userIds == null || userIds.isEmpty()) {
            return;
        }
        Map<String, Object> params = getHashMap(1);
        params.put("departmentId", departmentId);
        params.put("userIds", userIds);
        departmentUserDao.delete(params);
        departmentAdjustmentService.addUserDepartmentAdjustment(userIds, departmentId, adjustmentDepartmentName, USER_DEPARTMENT_TYPE_LEAVE);
    }

    @Override
    public void deleteUserByDepartmentId(String departmentId) {
        Map<String, Object> params = getHashMap(2);
        params.put("departmentId", departmentId);
        List<String> userIds = departmentUserDao.listUserId(params);
        deleteUserByDepartmentIdAndUserIds(departmentId, null, userIds);
    }

    @Override
    public void updateUserByDepartmentIdAndUserIds(String departmentId, List<String> userIds) {
        DepartmentSimpleDTO departmentSimpleDTO = departmentService.getSimple(departmentId);
        if (departmentSimpleDTO == null) {
            throw new SearchException("部门不存在");
        }
        Map<String, Object> params = getHashMap(2);
        params.put("departmentId", departmentId);
        List<String> existUserIds = departmentUserDao.listUserId(params);
        List<String> newUserIds = new ArrayList<>();
        for (String userId : userIds) {
            boolean isNewUser = true;
            for (String existUserId : existUserIds) {
                if (StringUtils.equals(userId, existUserId)) {
                    isNewUser = false;
                    break;
                }
            }
            if (isNewUser) {
                params.put("userId", userId);
                departmentUserDao.save(params);
                newUserIds.add(userId);
            }
        }
        departmentAdjustmentService.addUserDepartmentAdjustment(newUserIds, departmentId, departmentSimpleDTO.getDepartmentName(), USER_DEPARTMENT_TYPE_JOIN);
    }

    @Override
    public List<String> listUserIdByDepartmentId(String departmentId) {
        Map<String, Object> params = getHashMap(2);
        params.put("departmentId", departmentId);
        return departmentUserDao.listUserId(params);
    }

    @Override
    public List<String> listUserIdByDepartmentIds(List<String> departmentIds) {
        Map<String, Object> params = getHashMap(2);
        params.put("departmentIds", departmentIds);
        return departmentUserDao.listUserId(params);
    }
}
