package ink.wgink.module.group.dao;

import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SaveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.UpdateException;
import ink.wgink.pojo.bos.GroupBO;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.group.GroupDTO;
import ink.wgink.module.group.pojo.pos.GroupPO;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IGroupDao
 * @Description: 组
 * @Author: wenc
 * @Date: 2019/1/6 3:49 PM
 * @Version: 1.0
 **/
@Repository
public interface IGroupDao {

    /**
     * 新增组
     *
     * @param params
     * @throws SaveException
     */
    void saveGroup(Map<String, Object> params) throws SaveException;

    /**
     * 删除组
     *
     * @param params
     * @throws RemoveException
     */
    void removeGroup(Map<String, Object> params) throws RemoveException;

    /**
     * 修改组
     *
     * @param params
     * @throws UpdateException
     */
    void updateGroup(Map<String, Object> params) throws UpdateException;

    /**
     * 组列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<GroupDTO> listGroups(Map<String, Object> params) throws SearchException;

    /**
     * 组详情
     *
     * @param params
     * @return
     * @throws SearchException
     */
    GroupDTO getGroup(Map<String, Object> params) throws SearchException;

    /**
     * 组详情
     *
     * @param params
     * @return
     * @throws SearchException
     */
    GroupPO getGroupPO(Map<String, Object> params) throws SearchException;

    /**
     * 获取最后一个字组
     *
     * @param parentId
     * @return
     * @throws SearchException
     */
    GroupDTO getLastByParentId(String parentId) throws SearchException;

    /**
     * 组ZTree列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<ZTreeDTO> listZTreeGroups(Map<String, Object> params) throws SearchException;

    /**
     * 子列表个数
     *
     * @param id
     * @return
     * @throws SearchException
     */
    Integer countByParentId(String id) throws SearchException;

    /**
     * 删除角色组
     *
     * @param params
     * @throws RemoveException
     */
    void removeRoleGroup(Map<String, Object> params) throws RemoveException;

    /**
     * 添加角色组
     *
     * @param params
     * @throws SaveException
     */
    void saveRoleGroup(Map<String, Object> params) throws SaveException;

    /**
     * 通过用户获取用户组
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<GroupBO> listGroupBOByUser(Map<String, Object> params) throws SearchException;
}
