package ink.wgink.module.group.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.bos.GroupBO;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.group.GroupDTO;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.module.group.dao.IGroupDao;
import ink.wgink.module.group.service.IGroupService;
import ink.wgink.util.UUIDUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: GroupServiceImpl
 * @Description: 组
 * @Author: wenc
 * @Date: 2019/1/6 3:21 PM
 * @Version: 1.0
 **/
@Service
public class GroupServiceImpl extends DefaultBaseService implements IGroupService {

    @Autowired
    private IGroupDao groupDao;

    @Override
    public SuccessResult saveGroup(Map<String, Object> params) {
        String parentCode = null;
        String groupParentId = params.get("groupParentId").toString();
        if (!"0".equals(groupParentId)) {
            params.put("groupId", groupParentId);
            GroupDTO groupDTO = groupDao.getGroup(params);
            parentCode = groupDTO.getGroupCode();
        }
        String groupCode = getCode(parentCode, groupParentId);
        params.put("groupCode", groupCode);
        params.put("groupId", UUIDUtil.getUUID());
        setSaveInfo(params);
        groupDao.saveGroup(params);
        return new SuccessResult();
    }

    @Override
    public SuccessResult removeGroup(Map<String, Object> params) {
        params.put("groupIds", Arrays.asList(params.get("groupIds").toString().split("_")));
        setUpdateInfo(params);
        groupDao.removeGroup(params);
        return new SuccessResult();
    }

    @Override
    public SuccessResult updateGroup(Map<String, Object> params) {
        setUpdateInfo(params);
        groupDao.updateGroup(params);
        return new SuccessResult();
    }

    @Override
    public List<GroupDTO> listGroupAllJson(Map<String, Object> params) {
        return listGroupsAll(params);
    }

    @Override
    public List<GroupDTO> listGroupsAll(Map<String, Object> params) {
        List<GroupDTO> groupDTOs = groupDao.listGroups(params);
        listSubGroups(groupDTOs, params);
        return groupDTOs;
    }

    @Override
    public GroupDTO getGroup(Map<String, Object> params) {
        return groupDao.getGroup(params);
    }

    @Override
    public List<ZTreeDTO> listZTreeGroup(Map<String, Object> params) {
        List<ZTreeDTO> zTreeDTOs = groupDao.listZTreeGroups(params);
        for (ZTreeDTO zTreeDTO : zTreeDTOs) {
            Integer subCount = groupDao.countByParentId(zTreeDTO.getId());
            setZTreeInfo(zTreeDTO, subCount);
        }
        return zTreeDTOs;
    }

    @Override
    public SuccessResultList<List<GroupDTO>> listPageGroups(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<GroupDTO> groupDTOs = groupDao.listGroups(page.getParams());
        PageInfo<GroupDTO> pageInfo = new PageInfo<>(groupDTOs);
        return new SuccessResultList<>(groupDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    @Override
    public SuccessResult updateRoleGroup(Map<String, Object> params) {
        groupDao.removeRoleGroup(params);
        if (!StringUtils.isBlank(params.get("selectedGroupIds") == null ? null : params.get("selectedGroupIds").toString())) {
            String[] selectedUserIds = params.get("selectedGroupIds").toString().split("_");
            params.remove("selectedGroupIds");
            for (String userId : selectedUserIds) {
                params.put("groupId", userId);
                groupDao.saveRoleGroup(params);
            }
        }
        return new SuccessResult();
    }

    @Override
    public SuccessResult removeRoleGroup(Map<String, Object> params) {
        params.put("groupIds", Arrays.asList(params.get("groupIds").toString().split("_")));
        groupDao.removeRoleGroup(params);
        return new SuccessResult();
    }

    @Override
    public List<GroupBO> listGroupBOByUser(Map<String, Object> params) {
        return groupDao.listGroupBOByUser(params);
    }

    @Override
    public List<GroupDTO> listGroup(Map<String, Object> params) {
        return groupDao.listGroups(params);
    }

    /**
     * 递归查询子组
     *
     * @param groupDTOs
     * @param params
     */
    private void listSubGroups(List<GroupDTO> groupDTOs, Map<String, Object> params) {
        for (GroupDTO groupDTO : groupDTOs) {
            params.put("groupParentId", groupDTO.getGroupId());
            List<GroupDTO> subGroupDTOs = groupDao.listGroups(params);
            groupDTO.setSubGroups(subGroupDTOs);
            listSubGroups(subGroupDTOs, params);
        }
    }

    /**
     * 获取code
     *
     * @param parentCode
     * @param parentId
     * @return
     */
    private String getCode(String parentCode, String parentId) {
        GroupDTO groupDTO = groupDao.getLastByParentId(parentId);
        String code = groupDTO != null ? groupDTO.getGroupCode() : "0000";
        return super.getNewCode(code, parentCode);
    }

}
