package ink.wgink.module.menu.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.module.menu.dao.IMenuDao;
import ink.wgink.module.menu.pojo.vos.MenuVO;
import ink.wgink.module.menu.service.IMenuService;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.map.HashMapUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @ClassName: MenuServiceImpl
 * @Description: 菜单
 * @Author: wenc
 * @Date: 2018/12/27 10:18 PM
 * @Version: 1.0
 **/
@Service
public class MenuServiceImpl extends DefaultBaseService implements IMenuService {

    @Autowired
    private IMenuDao menuDao;
//    @Autowired
//    private IRoleDao roleDao;
//    @Autowired
//    private IOauthClientService oauthClientService;
//    @Autowired
//    private SecurityComponent securityComponent;
//    @Autowired
//    private IUserService userService;

    @Override
    public MenuDTO get(Map<String, Object> params) {
        return menuDao.get(params);
    }

    @Override
    public List<MenuDTO> list(Map<String, Object> params) {
        List<MenuDTO> srcMenuDTOs = menuDao.list(params);
        List<MenuDTO> desMenuDTOs = Arrays.asList(new MenuDTO[srcMenuDTOs.size()]);
        Collections.copy(desMenuDTOs, srcMenuDTOs);
        return srcMenuDTOs;
    }

    @Override
    public List<MenuDTO> listByParentId(String menuParentId) {
        Map<String, Object> params = getHashMap(2);
        params.put("menuParentId", menuParentId);
        return list(params);
    }

    @Override
    public List<MenuDTO> listByIds(List<String> menuIds) {
        Map<String, Object> params = getHashMap(2);
        params.put("menuIds", menuIds);
        return list(params);
    }

    @Override
    public List<MenuDTO> listByParentIdAndIds(String menuParentId, List<String> menuIds) {
        Map<String, Object> params = getHashMap(4);
        params.put("menuParentId", menuParentId);
        params.put("menuIds", menuIds);
        return list(params);
    }

    @Override
    public List<MenuDTO> listAll(Map<String, Object> params) {
        List<MenuDTO> menuDTOs = list(params);
        listSub(menuDTOs);
        return menuDTOs;
    }

    @Override
    public List<MenuDTO> listAllByParentId(String menuParentId) {
        List<MenuDTO> menuDTOs = listByParentId(menuParentId);
        listSub(menuDTOs);
        return menuDTOs;
    }

    @Override
    public List<MenuDTO> listAllByParentIdAndIds(String menuParentId, List<String> menuIds) {
        List<MenuDTO> menuDTOs = listByParentIdAndIds(menuParentId, menuIds);
        listSubByIds(menuDTOs, menuIds);
        return menuDTOs;
    }

    @Override
    public List<ZTreeDTO> listZTree(Map<String, Object> params) {
        List<ZTreeDTO> zTreeDTOs = menuDao.listZTree(params);
        for (ZTreeDTO zTreeDTO : zTreeDTOs) {
            Integer subCount = menuDao.countByParentId(zTreeDTO.getId());
            setZTreeInfo(zTreeDTO, subCount);
        }
        return zTreeDTOs;
    }

    @Override
    public SuccessResultList<List<MenuDTO>> listPage(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<MenuDTO> menuDTOs = list(page.getParams());
        PageInfo<MenuDTO> pageInfo = new PageInfo<>(menuDTOs);
        return new SuccessResultList<>(menuDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    @Override
    public synchronized SuccessResult save(MenuVO menuVO) {
        Map<String, Object> params = HashMapUtil.beanToMap(menuVO);
        String parentCode = null;
        String menuParentId = params.get("menuParentId").toString();
        if (!"0".equals(menuParentId)) {
            params.put("menuId", menuParentId);
            MenuDTO menuDTO = menuDao.get(params);
            parentCode = menuDTO.getMenuCode();
        }
        String menuCode = getCode(parentCode, menuParentId);
        params.put("menuCode", menuCode);
        params.put("menuId", UUIDUtil.getUUID());
        setSaveInfo(params);
        menuDao.save(params);
        return new SuccessResult();
    }

    @Override
    public SuccessResult remove(List<String> ids) {
        Map<String, Object> params = getHashMap(2);
        params.put("menuIds", ids);
        setUpdateInfo(params);
        menuDao.remove(params);
        return new SuccessResult();
    }

    @Override
    public SuccessResult update(String menuId, MenuVO menuVO) {
        Map<String, Object> params = HashMapUtil.beanToMap(menuVO);
        params.put("menuId", menuId);
        setUpdateInfo(params);
        menuDao.update(params);
        return new SuccessResult();
    }
//    @Override
//    public SuccessResultData<List<MenuDTO>> listMenuByClientId(Map<String, Object> params) {
//        OauthClientDTO oauthClientDTO = oauthClientService.getOauthClient(params);
//        if (oauthClientDTO == null) {
//            throw new SearchException("客户端不存在");
//        }
//        String menuId = oauthClientDTO.getMenuId();
//        if (StringUtils.isBlank(menuId)) {
//            return new SuccessResultData<>(new ArrayList<>());
//        }
//        // 获取全部菜单
//        params.clear();
//        params.put("menuParentId", menuId);
//        params.put("menuStatus", 0);
//        if (!SecurityComponent.USERNAME_ADMIN.equals(securityComponent.getCurrentUsername())) {
//            log.debug("非管理员第三方查询菜单");
//            params.put("menuIds", listUserMenuId());
//        }
//        List<MenuDTO> menuDTOs = listMenusAll(params);
//        return new SuccessResultData<>(menuDTOs);
//    }

//    @Override
//    public List<String> listUserMenuId() {
//        List<RoleMenuBO> roleMenuBOs = listRoleMenu();
//        List<String> menuIds = new ArrayList<>();
//        for (RoleMenuBO roleMenuBO : roleMenuBOs) {
//            menuIds.add(roleMenuBO.getMenuId());
//        }
//        return menuIds;
//    }

//    @Override
//    public SuccessResultData<List<MenuDTO>> listMenuByClientIdAndUserId(String clientId, String userId) {
//        Map<String, Object> params = new HashMap<>(1);
//        params.put("clientId", clientId);
//        OauthClientDTO oauthClientDTO = oauthClientService.getOauthClient(params);
//        if (oauthClientDTO == null) {
//            throw new SearchException("客户端不存在");
//        }
//        String menuId = oauthClientDTO.getMenuId();
//        if (StringUtils.isBlank(menuId)) {
//            return new SuccessResultData<>(new ArrayList<>());
//        }
//        List<String> userMenuIds;
//        // 如果是管理员获取管理员配置的菜单，管理员的ID是1
//        if (StringUtils.equalsIgnoreCase(userId, "1")) {
//            userMenuIds = listAdminMenuId();
//        } else {
//            params.put("userId", userId);
//            params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_MENU);
//            userMenuIds = listMenuIdByUser(params);
//        }
//        // 获取全部菜单
//        params.clear();
//        params.put("menuParentId", menuId);
//        params.put("menuStatus", 0);
//        params.put("menuIds", userMenuIds);
//        List<MenuDTO> menuDTOs = listMenusAll(params);
//        return new SuccessResultData<>(menuDTOs);
//    }

    @Override
    public List<String> listMenuIdByUser(Map<String, Object> params) {
        return menuDao.listIdByUser(params);
    }

    /**
     * 获取角色菜单列表
     *
     * @return
     * @throws SearchException
     */
//    private List<RoleMenuBO> listRoleMenu() {
//        Map<String, Object> params = new HashMap<>(2);
//        List<String> listRoleIds = securityComponent.listRoleIds();
//        params.put("listRoleIds", listRoleIds);
//        params.put("roleType", IRoleService.ROLE_MENU);
//        List<RoleMenuBO> roleMenuBOs = roleDao.listRoleMenuInfo(params);
//        return roleMenuBOs;
//    }

    /**
     * 管理员菜单列表
     *
     * @return
     * @throws SearchException
     */
//    private List<RoleMenuBO> listAdminMenu() {
//        Map<String, Object> params = new HashMap<>(4);
//        params.put("listRoleIds", Arrays.asList(SecurityComponent.USERNAME_ADMIN));
//        params.put("roleType", IRoleService.ROLE_MENU);
//        List<RoleMenuBO> roleMenuBOs = roleDao.listRoleMenuInfo(params);
//        return roleMenuBOs;
//    }

    /**
     * 管理员菜单ID列表
     *
     * @return
     */
//    private List<String> listAdminMenuId() {
//        List<RoleMenuBO> roleMenuBOs = listAdminMenu();
//        List<String> menuIds = new ArrayList<>();
//        for (RoleMenuBO roleMenuBO : roleMenuBOs) {
//            menuIds.add(roleMenuBO.getMenuId());
//        }
//        return menuIds;
//    }

    /**
     * 递归查询子菜单
     *
     * @param menuDTOs
     */
    private void listSub(List<MenuDTO> menuDTOs) {
        if (menuDTOs.isEmpty()) {
            return;
        }
        for (MenuDTO menuDTO : menuDTOs) {
            List<MenuDTO> subMenuDTOs = listByParentId(menuDTO.getMenuId());
            menuDTO.setSubMenus(subMenuDTOs);
            if (!subMenuDTOs.isEmpty()) {
                menuDTO.setParent(true);
            }
            listSub(subMenuDTOs);
        }
    }

    private void listSubByIds(List<MenuDTO> menuDTOs, List<String> menuIds) {
        if (menuDTOs.isEmpty()) {
            return;
        }
        for (MenuDTO menuDTO : menuDTOs) {
            List<MenuDTO> subMenuDTOs = listByParentIdAndIds(menuDTO.getMenuId(), menuIds);
            menuDTO.setSubMenus(subMenuDTOs);
            if (!subMenuDTOs.isEmpty()) {
                menuDTO.setParent(true);
            }
            listSubByIds(subMenuDTOs, menuIds);
        }
    }

    /**
     * 子菜单权限
     *
     * @param menuDTOs
     * @param params
     * @param roleMenuBOs
     * @throws SearchException
     */
//    private void listSubMenus(List<MenuDTO> menuDTOs, Map<String, Object> params, List<RoleMenuBO> roleMenuBOs) {
//        for (MenuDTO menuDTO : menuDTOs) {
//            if (roleMenuBOs != null) {
//                boolean hasRight = false;
//                for (RoleMenuBO roleMenuBO : roleMenuBOs) {
//                    if (menuDTO.getMenuId().equals(roleMenuBO.getMenuId())) {
//                        hasRight = true;
//                        break;
//                    }
//                }
//                menuDTO.setRight(hasRight);
//            }
//            params.put("menuParentId", menuDTO.getMenuId());
//            List<MenuDTO> subMenuDTOs = new ArrayList<>(menuDao.listMenus(params));
//            menuDTO.setSubMenus(subMenuDTOs);
//            if (!subMenuDTOs.isEmpty()) {
//                menuDTO.setParent(true);
//            } else {
//                menuDTO.setParent(false);
//            }
//            listSubMenus(subMenuDTOs, params, roleMenuBOs);
//        }
//    }

    /**
     * 获取code
     *
     * @param parentCode
     * @param parentId
     * @return
     * @throws SearchException
     */
    private String getCode(String parentCode, String parentId) {
        MenuDTO menuDTO = menuDao.getLastByParentId(parentId);
        String code = menuDTO != null ? menuDTO.getMenuCode() : "0000";
        return super.getNewCode(code, parentCode);
    }

}
