package ink.wgink.module.menu.startup;

import ink.wgink.interfaces.dictionary.IDictionaryCheckService;
import ink.wgink.interfaces.menu.IMenuBaseService;
import ink.wgink.interfaces.user.IUserCheckService;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import ink.wgink.module.menu.dao.IMenuDao;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.date.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: ServiceMenuStartUp
 * @Description: 菜单业务启动
 * @Author: wanggeng
 * @Date: 2021/2/10 1:12 下午
 * @Version: 1.0
 */
@Component
public class ServiceMenuStartUp implements ApplicationRunner {
    private static final Logger LOG = LoggerFactory.getLogger(ServiceMenuStartUp.class);
    @Autowired
    private IMenuDao menuDao;
    @Autowired(required = false)
    private IUserCheckService userCheckService;
    @Autowired(required = false)
    private IDictionaryCheckService dictionaryCheckService;

    @Override
    public void run(ApplicationArguments args) throws Exception {
        initTable();
    }

    /**
     * 建表
     */
    private void initTable() {
        LOG.debug("创建 sys_menu 表");
        menuDao.createTable();
        initMenu();
    }

    private void initMenu() {
        Map<String, Object> params = new HashMap<>(20);
        String gmt = DateUtil.getTime();
        params.put("gmtCreate", gmt);
        params.put("gmtModified", gmt);
        params.put("creator", "1");
        params.put("modifier", "1");
        params.put("isDelete", 0);

        params.put("menuCode", "0001");
        MenuDTO menuDTO = menuDao.getSimple(params);
        String menuId = IMenuBaseService.MENU_UNIFIED_USER;
        if (menuDTO == null) {
            params.put("menuId", menuId);
            params.put("menuParentId", "0");
            params.put("menuName", "统一用户系统");
            params.put("menuSummary", "统一用户系统");
            params.put("menuUrl", "javascript:void(0);");
            params.put("menuType", "1");
            params.put("menuIcon", "fa-icon-color-white fa fa-cogs");
            params.put("menuOrder", "1");
            params.put("menuStatus", "0");
            params.put("openType", "1");
            menuDao.save(params);
        }
        initSystemManage(params, menuId);
        initUserPermissionManage(params, menuId);
    }

    private void initSystemManage(Map<String, Object> params, String menuParentId) {
        LOG.debug("初始化菜单：系统管理");
        params.remove("menuId");
        params.put("menuCode", "00010001");
        MenuDTO menuDTO = menuDao.getSimple(params);
        String menuId;
        if (menuDTO == null) {
            menuId = UUIDUtil.getUUID();
            params.put("menuId", menuId);
            params.put("menuParentId", menuParentId);
            params.put("menuName", "系统管理");
            params.put("menuSummary", "系统管理");
            params.put("menuUrl", "javascript:void(0);");
            params.put("menuType", "1");
            params.put("menuIcon", "fa-icon-color-white fa fa-cogs");
            params.put("menuOrder", "1");
            params.put("menuStatus", "0");
            params.put("openType", "1");
            menuDao.save(params);
        } else {
            menuId = menuDTO.getMenuId();
        }
        initMenuManage(params, menuId);
        initDataManager(params, menuId);
        initAreaManager(params, menuId);
    }

    /**
     * 菜单管理
     *
     * @param params
     * @param menuParentId
     */
    private void initMenuManage(Map<String, Object> params, String menuParentId) {
        LOG.debug("初始化菜单：菜单管理");
        params.remove("menuId");
        params.put("menuCode", "000100010001");
        MenuDTO menuDTO = menuDao.getSimple(params);
        if (menuDTO == null) {
            params.put("menuId", UUIDUtil.getUUID());
            params.put("menuParentId", menuParentId);
            params.put("menuName", "菜单管理");
            params.put("menuSummary", "菜单管理");
            params.put("menuUrl", "/route/menu/list-tree");
            params.put("menuType", "1");
            params.put("menuIcon", "fa-icon-color-white fa fa-list");
            params.put("menuOrder", "1");
            params.put("menuStatus", "0");
            params.put("openType", "1");
            menuDao.save(params);
        }
    }

    /**
     * 初始化数据字典
     *
     * @param params
     * @param menuParentId
     */
    public void initDataManager(Map<String, Object> params, String menuParentId) {
        if (dictionaryCheckService == null) {
            return;
        }
        LOG.debug("初始化菜单：数据字典");
        params.remove("menuId");
        params.put("menuCode", "000100010002");
        MenuDTO menuDTO = menuDao.getSimple(params);
        if (menuDTO == null) {
            params.put("menuId", UUIDUtil.getUUID());
            params.put("menuParentId", menuParentId);
            params.put("menuName", "数据字典");
            params.put("menuSummary", "数据字典");
            params.put("menuUrl", "/route/data/list-tree");
            params.put("menuType", "1");
            params.put("menuIcon", "fa-icon-color-white fa fa-address-book");
            params.put("menuOrder", "2");
            params.put("menuStatus", "0");
            params.put("openType", "1");
            menuDao.save(params);
        }
    }

    /**
     * 初始化地区字典
     *
     * @param params
     * @param menuParentId
     */
    public void initAreaManager(Map<String, Object> params, String menuParentId) {
        if (dictionaryCheckService == null) {
            return;
        }
        LOG.debug("初始化菜单：地区字典");
        params.remove("menuId");
        params.put("menuCode", "000100010003");
        MenuDTO menuDTO = menuDao.getSimple(params);
        if (menuDTO == null) {
            params.put("menuId", UUIDUtil.getUUID());
            params.put("menuParentId", menuParentId);
            params.put("menuName", "区域字典");
            params.put("menuSummary", "区域字典");
            params.put("menuUrl", "/route/area/list-tree");
            params.put("menuType", "1");
            params.put("menuIcon", "fa-icon-color-white fa fa-map-o");
            params.put("menuOrder", "3");
            params.put("menuStatus", "0");
            params.put("openType", "1");
            menuDao.save(params);
        }
    }

    /**
     * 用户权限管理
     *
     * @param params
     * @param menuParentId
     */
    private void initUserPermissionManage(Map<String, Object> params, String menuParentId) {
        LOG.debug("初始化菜单：用户与权限");
        params.remove("menuId");
        params.put("menuCode", "00010002");
        MenuDTO menuDTO = menuDao.getSimple(params);
        String menuId;
        if (menuDTO == null) {
            menuId = UUIDUtil.getUUID();
            params.put("menuId", menuId);
            params.put("menuParentId", menuParentId);
            params.put("menuName", "用户与权限");
            params.put("menuSummary", "用户与权限");
            params.put("menuUrl", "javascript:void(0);");
            params.put("menuType", "1");
            params.put("menuIcon", "fa-icon-color-white fa fa-users");
            params.put("menuOrder", "2");
            params.put("menuStatus", "0");
            params.put("openType", "1");
            menuDao.save(params);
        } else {
            menuId = menuDTO.getMenuId();
        }
        initUserManage(params, menuId);
    }

    /**
     * 用户管理
     *
     * @param params
     * @param menuParentId
     */
    private void initUserManage(Map<String, Object> params, String menuParentId) {
        if (userCheckService == null) {
            return;
        }
        LOG.debug("初始化菜单：用户管理");
        params.remove("menuId");
        params.put("menuCode", "000100020001");
        MenuDTO menuDTO = menuDao.getSimple(params);
        String menuId = UUIDUtil.getUUID();
        if (menuDTO == null) {
            params.put("menuId", menuId);
            params.put("menuParentId", menuParentId);
            params.put("menuName", "用户管理");
            params.put("menuSummary", "用户管理");
            params.put("menuUrl", "/route/user/list");
            params.put("menuType", "1");
            params.put("menuIcon", "fa-icon-color-white fa fa-address-book");
            params.put("menuOrder", "1");
            params.put("menuStatus", "0");
            params.put("openType", "1");
            menuDao.save(params);
        }
    }

    private void initMenuDepartment() {
        LOG.debug("初始化菜单：组织机构管理");
        LOG.debug("初始化菜单：用户组管理");
        LOG.debug("初始化菜单：职位管理");
        LOG.debug("初始化菜单：权限管理");
        LOG.debug("初始化菜单：角色管理");
    }

    private void initMenuGroup() {

    }

    private void initMenuPosition() {

    }

    private void initMenuPermission() {

    }

    private void initMenuRole() {

    }
}
