package ink.wgink.module.position.service;

import ink.wgink.interfaces.position.IPositionBaseService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.bos.PositionBO;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.position.PositionDTO;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IPositionService
 * @Description: 职位
 * @Author: admin
 * @Date: 2019-04-11 22:41:20
 * @Version: 1.0
 **/
public interface IPositionService extends IPositionBaseService {

    /**
     * 新增职位
     *
     * @param params
     * @return
     */
    SuccessResult save(Map<String, Object> params);

    /**
     * 删除职位
     *
     * @param params
     * @return
     * @throws RemoveException
     */
    SuccessResult remove(Map<String, Object> params);

    /**
     * 修改职位
     *
     * @param params
     * @return
     */
    SuccessResult update(Map<String, Object> params);

    /**
     * 全部职位JSON列表
     *
     * @param params
     * @return
     */
    List<PositionDTO> listAllJson(Map<String, Object> params);

    /**
     * 组列表，递归获取全部内容
     *
     * @param params
     * @return
     */
    List<PositionDTO> listAll(Map<String, Object> params);

    /**
     * 职位详情
     *
     * @param params
     * @return
     */
    PositionDTO get(Map<String, Object> params);

    /**
     * 职位zTree列表
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(Map<String, Object> params);

    /**
     * 职位分页列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<PositionDTO>> listPage(ListPage page);

    /**
     * 职位列表，无分页
     *
     * @param params
     * @return
     */
    List<PositionDTO> list(Map<String, Object> params);

    /**
     * 用户职位列表
     *
     * @param params
     * @return
     */
    List<PositionBO> listBOByUser(Map<String, Object> params);

    /**
     * 用户职位列表
     *
     * @param params
     * @return
     */
    List<PositionDTO> listUser(Map<String, Object> params);

    /**
     * 统计职位
     *
     * @param params
     * @return
     */
    Integer count(Map<String, Object> params);
}
