package ink.wgink.module.position.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.bos.PositionBO;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.position.PositionDTO;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.module.position.dao.IPositionDao;
import ink.wgink.module.position.service.IPositionService;
import ink.wgink.util.UUIDUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: PositionServiceImpl
 * @Description: 职位
 * @Author: admin
 * @Date: 2019-04-11 22:41:20
 * @Version: 1.0
 **/
@Service
public class PositionServiceImpl extends DefaultBaseService implements IPositionService {

    @Autowired
    private IPositionDao positionDao;

    @Override
    public SuccessResult save(Map<String, Object> params) {
        String parentCode = null;
        String positionParentId = params.get("positionParentId").toString();
        if (!"0".equals(positionParentId)) {
            params.put("positionId", positionParentId);
            PositionDTO positionDTO = positionDao.get(params);
            parentCode = positionDTO.getPositionCode();
        }
        String positionCode = getCode(parentCode, positionParentId);
        params.put("positionCode", positionCode);
        params.put("positionId", UUIDUtil.getUUID());
        setSaveInfo(params);
        positionDao.save(params);
        return new SuccessResult();
    }

    @Override
    public SuccessResult remove(Map<String, Object> params) {
        params.put("positionIds", Arrays.asList(params.get("positionIds").toString().split("_")));
        setUpdateInfo(params);
        positionDao.remove(params);
        return new SuccessResult();
    }

    @Override
    public SuccessResult update(Map<String, Object> params) {
        setUpdateInfo(params);
        positionDao.update(params);
        return new SuccessResult();
    }

    @Override
    public List<PositionDTO> listAllJson(Map<String, Object> params) {
        return listAll(params);
    }

    @Override
    public List<PositionDTO> listAll(Map<String, Object> params) {
        List<PositionDTO> positionDTOs = positionDao.list(params);
        listSub(positionDTOs, params);
        return positionDTOs;
    }

    @Override
    public PositionDTO get(Map<String, Object> params) {
        return positionDao.get(params);
    }

    @Override
    public List<ZTreeDTO> listZTree(Map<String, Object> params) {
        List<ZTreeDTO> zTreeDTOs = positionDao.listZTree(params);
        for (ZTreeDTO zTreeDTO : zTreeDTOs) {
            Integer subCount = positionDao.countByParentId(zTreeDTO.getId());
            setZTreeInfo(zTreeDTO, subCount);
        }
        return zTreeDTOs;
    }

    @Override
    public SuccessResultList<List<PositionDTO>> listPage(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<PositionDTO> positionDTOs = positionDao.list(page.getParams());
        PageInfo<PositionDTO> pageInfo = new PageInfo<>(positionDTOs);
        return new SuccessResultList<>(positionDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    @Override
    public List<PositionDTO> list(Map<String, Object> params) {
        return positionDao.list(params);
    }

    @Override
    public List<PositionBO> listBOByUser(Map<String, Object> params) {
        return positionDao.listBOByUser(params);
    }

    @Override
    public List<PositionDTO> listUser(Map<String, Object> params) {
        return positionDao.listUser(params);
    }

    @Override
    public Integer count(Map<String, Object> params) {
        Integer positionCount = positionDao.count(params);
        return positionCount == null ? 0 : positionCount;
    }

    /**
     * 递归查询子职位
     *
     * @param positionDTOs
     * @param params
     */
    private void listSub(List<PositionDTO> positionDTOs, Map<String, Object> params) {
        for (PositionDTO positionDTO : positionDTOs) {
            params.put("positionParentId", positionDTO.getPositionId());
            List<PositionDTO> subPositionDTOs = positionDao.list(params);
            positionDTO.setSubPositions(subPositionDTOs);
            listSub(subPositionDTOs, params);
        }
    }

    /**
     * 获取code
     *
     * @param parentCode
     * @param parentId
     * @return
     */
    private String getCode(String parentCode, String parentId) {
        PositionDTO positionDTO = positionDao.getLastByParentId(parentId);
        String code = positionDTO != null ? positionDTO.getPositionCode() : "0000";
        return super.getNewCode(code, parentCode);
    }

}
