package ink.wgink.module.role.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.common.enums.RoleDataAuthorityEnum;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SaveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.UpdateException;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.bos.*;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.role.RoleDTO;
import ink.wgink.pojo.dtos.role.RoleDataAuthorityDTO;
import ink.wgink.pojo.dtos.role.RoleDataAuthorityTypeDTO;
import ink.wgink.pojo.dtos.role.RolePermissionDTO;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.module.department.service.IDepartmentUserService;
import ink.wgink.module.permission.enums.PermissionTypeEnum;
import ink.wgink.module.permission.service.IPermissionService;
import ink.wgink.module.role.dao.IRoleDao;
import ink.wgink.module.role.pojo.vos.RoleDataAuthorityVO;
import ink.wgink.module.role.pojo.vos.RoleUserAuthorityVO;
import ink.wgink.module.role.service.IRolePermissionService;
import ink.wgink.module.role.service.IRoleService;
import ink.wgink.module.user.service.IUserService;
import ink.wgink.util.UUIDUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: RoleServiceImpl
 * @Description: 角色
 * @Author: WangGeng
 * @Date: 2021/1/31 18:51
 * @Version: 1.0
 **/
@Service
public class RoleServiceImpl extends DefaultBaseService implements IRoleService {

    @Autowired
    private IRoleDao roleDao;
    @Autowired
    private IUserService userService;
    @Autowired
    private IDepartmentUserService departmentUserService;
    @Autowired
    private IPermissionService permissionService;
    @Autowired
    private IRolePermissionService rolePermissionService;

    @Override
    public SuccessResult saveRole(Map<String, Object> params) throws SaveException, SearchException {
        String parentCode = null;
        String roleParentId = params.get("roleParentId").toString();
        if (!"0".equals(roleParentId)) {
            params.put("roleId", roleParentId);
            RoleDTO roleDTO = roleDao.getRole(params);
            parentCode = roleDTO.getRoleCode();
        }
        String roleCode = getCode(parentCode, roleParentId);
        params.put("roleCode", roleCode);
        params.put("roleId", UUIDUtil.getUUID());
        setSaveInfo(params);
        roleDao.saveRole(params);
        return new SuccessResult();
    }

    @Override
    public SuccessResult removeRole(Map<String, Object> params) throws RemoveException {
        params.put("roleIds", Arrays.asList(params.get("roleIds").toString().split("_")));
        setUpdateInfo(params);
        roleDao.removeRole(params);
        return new SuccessResult();
    }

    @Override
    public SuccessResult updateRole(Map<String, Object> params) throws UpdateException {
        setUpdateInfo(params);
        roleDao.updateRole(params);
        return new SuccessResult();
    }

    @Override
    public List<RoleDTO> listRolesAllJson(Map<String, Object> params) throws SearchException {
        return listRolesAll(params);
    }

    @Override
    public RoleDTO getRole(Map<String, Object> params) throws SearchException {
        return roleDao.getRole(params);
    }

    @Override
    public List<RoleDTO> listRolesAll(Map<String, Object> params) throws SearchException {
        List<RoleDTO> roleDTOs = roleDao.listRole(params);
        listSubRoles(roleDTOs, params);
        return roleDTOs;
    }

    @Override
    public List<ZTreeDTO> listZTreeRole(Map<String, Object> params) throws SearchException {
        List<ZTreeDTO> zTreeDTOs = roleDao.listZTreeRoles(params);
        for (ZTreeDTO zTreeDTO : zTreeDTOs) {
            Integer subCount = roleDao.countByParentId(zTreeDTO.getId());
            setZTreeInfo(zTreeDTO, subCount);
        }
        return zTreeDTOs;
    }

    @Override
    public SuccessResultList<List<RoleDTO>> listPageRole(ListPage page) throws SearchException {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<RoleDTO> roleDTOs = roleDao.listRole(page.getParams());
        PageInfo<RoleDTO> pageInfo = new PageInfo<>(roleDTOs);
        return new SuccessResultList<>(roleDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    @Override
    public SuccessResult authorization(Map<String, Object> params) throws UpdateException, RemoveException, SaveException {
        List<String> menuIds = new ArrayList<>();
        if (!StringUtils.isBlank(params.get("menuIds") == null ? null : params.get("menuIds").toString())) {
            menuIds.addAll(Arrays.asList(params.get("menuIds").toString().trim().split("_")));
        }
        String type = params.get("type").toString();
        if (IRoleService.AUTH_TYPE_INSERT.equals(type)) {
            params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_INSERT);
        } else if (IRoleService.AUTH_TYPE_DELETE.equals(type)) {
            params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_DELETE);
        } else if (IRoleService.AUTH_TYPE_UPDATE.equals(type)) {
            params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_UPDATE);
        } else if (IRoleService.AUTH_TYPE_QUERY.equals(type)) {
            params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_QUERY);
        } else {
            params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_MENU);
        }
        LOG.debug("删除原有角色和菜单信息");
        roleDao.removeRoleMenuInfo(params);
        LOG.debug("新增角色菜单信息");
        for (String menuId : menuIds) {
            params.put("menuId", menuId);
            roleDao.saveRoleMenuInfo(params);
        }
        return new SuccessResult();
    }

    @Override
    public List<RoleBO> listRoleBOByUser(Map<String, Object> params) throws SearchException {
        return roleDao.listRoleBOByUser(params);
    }

    @Override
    public List<RoleBO> listRoleBOByGroup(Map<String, Object> params) throws SearchException {
        return roleDao.listRoleBOByGroup(params);
    }

    @Override
    public List<RoleMenuBO> listRoleMenuDetail(Map<String, Object> params) throws SearchException {
        return roleDao.listRoleMenuDetail(params);
    }

    @Override
    public RoleBO getRoleBOByRoleId(String roleId) throws SearchException {
        Map<String, Object> params = new HashMap<>(0);
        params.put("roleId", roleId);
        List<RoleMenuBO> roleMenuBOs = listRoleMenuDetail(params);

        List<RoleMenuBO> apiSaveMenu = new ArrayList<>(0);
        List<RoleMenuBO> apiDeleteMenu = new ArrayList<>(0);
        List<RoleMenuBO> apiUpdateMenu = new ArrayList<>(0);
        List<RoleMenuBO> apiQueryMenu = new ArrayList<>(0);

        List<RoleMenuBO> resourceSaveMenu = new ArrayList<>(0);
        List<RoleMenuBO> resourceDeleteMenu = new ArrayList<>(0);
        List<RoleMenuBO> resourceUpdateMenu = new ArrayList<>(0);
        List<RoleMenuBO> resourceQueryMenu = new ArrayList<>(0);

        List<RoleMenuBO> routeSaveMenu = new ArrayList<>(0);
        List<RoleMenuBO> routeUpdateMenu = new ArrayList<>(0);
        List<RoleMenuBO> routeQueryMenu = new ArrayList<>(0);

        List<PermissionBO> permissionInsert = new ArrayList<>(0);
        List<PermissionBO> permissionDelete = new ArrayList<>(0);
        List<PermissionBO> permissionUpdate = new ArrayList<>(0);
        List<PermissionBO> permissionQuery = new ArrayList<>(0);

        LOG.debug("角色菜单权限");
        for (RoleMenuBO roleMenuBO : roleMenuBOs) {
            roleMenuBO.setApiPrefix(roleMenuBO.getApiPrefix());
            roleMenuBO.setResourcePrefix(roleMenuBO.getResourcePrefix());
            roleMenuBO.setRoutePrefix(roleMenuBO.getRoutePrefix());
            if (IRoleService.ROLE_INSERT.equals(roleMenuBO.getRoleType())) {
                apiSaveMenu.add(roleMenuBO);
                resourceSaveMenu.add(roleMenuBO);
                routeSaveMenu.add(roleMenuBO);
            } else if (IRoleService.ROLE_DELETE.equals(roleMenuBO.getRoleType())) {
                apiDeleteMenu.add(roleMenuBO);
                resourceDeleteMenu.add(roleMenuBO);
            } else if (IRoleService.ROLE_UPDATE.equals(roleMenuBO.getRoleType())) {
                apiUpdateMenu.add(roleMenuBO);
                resourceUpdateMenu.add(roleMenuBO);
                routeUpdateMenu.add(roleMenuBO);
            } else if (IRoleService.ROLE_QUERY.equals(roleMenuBO.getRoleType())) {
                apiQueryMenu.add(roleMenuBO);
                resourceQueryMenu.add(roleMenuBO);
                routeQueryMenu.add(roleMenuBO);
            }
        }
        LOG.debug("角色接口权限");
        setApiPermission(roleId, permissionInsert, permissionDelete, permissionUpdate, permissionQuery);
        RoleBO roleBO = new RoleBO();
        roleBO.setRoleId(roleId);
        roleBO.setApiSaveMenu(apiSaveMenu);
        roleBO.setApiDeleteMenu(apiDeleteMenu);
        roleBO.setApiUpdateMenu(apiUpdateMenu);
        roleBO.setApiQueryMenu(apiQueryMenu);
        roleBO.setResourceSaveMenu(resourceSaveMenu);
        roleBO.setResourceDeleteMenu(resourceDeleteMenu);
        roleBO.setResourceUpdateMenu(resourceUpdateMenu);
        roleBO.setResourceQueryMenu(resourceQueryMenu);
        roleBO.setRouteSaveMenu(routeSaveMenu);
        roleBO.setRouteUpdateMenu(routeUpdateMenu);
        roleBO.setRouteQueryMenu(routeQueryMenu);
        roleBO.setPermissionInsert(permissionInsert);
        roleBO.setPermissionDelete(permissionDelete);
        roleBO.setPermissionUpdate(permissionUpdate);
        roleBO.setPermissionQuery(permissionQuery);
        return roleBO;
    }

    /**
     * 设置API权限
     *
     * @param roleId
     * @param permissionInsert
     * @param permissionDelete
     * @param permissionUpdate
     * @param permissionQuery
     */
    private void setApiPermission(String roleId,
                                  List<PermissionBO> permissionInsert,
                                  List<PermissionBO> permissionDelete,
                                  List<PermissionBO> permissionUpdate,
                                  List<PermissionBO> permissionQuery) {
        List<RolePermissionDTO> permissionRoleDTOs = rolePermissionService.listByRoleId(roleId);
        for (RolePermissionDTO permissionRoleDTO : permissionRoleDTOs) {
            if (StringUtils.equals(PermissionTypeEnum.PERMISSION_INSERT.getValue(), permissionRoleDTO.getPermissionType())) {
                PermissionBO permissionBO = new PermissionBO();
                permissionBO.setPermissionType(permissionRoleDTO.getPermissionType());
                permissionBO.setPermissionUrl(permissionRoleDTO.getPermissionUrl());
                permissionInsert.add(permissionBO);
            } else if (StringUtils.equals(PermissionTypeEnum.PERMISSION_DELETE.getValue(), permissionRoleDTO.getPermissionType())) {
                PermissionBO permissionBO = new PermissionBO();
                permissionBO.setPermissionType(permissionRoleDTO.getPermissionType());
                permissionBO.setPermissionUrl(permissionRoleDTO.getPermissionUrl());
                permissionDelete.add(permissionBO);
            } else if (StringUtils.equals(PermissionTypeEnum.PERMISSION_UPDATE.getValue(), permissionRoleDTO.getPermissionType())) {
                PermissionBO permissionBO = new PermissionBO();
                permissionBO.setPermissionType(permissionRoleDTO.getPermissionType());
                permissionBO.setPermissionUrl(permissionRoleDTO.getPermissionUrl());
                permissionUpdate.add(permissionBO);
            } else if (StringUtils.equals(PermissionTypeEnum.PERMISSION_QUERY.getValue(), permissionRoleDTO.getPermissionType())) {
                PermissionBO permissionBO = new PermissionBO();
                permissionBO.setPermissionType(permissionRoleDTO.getPermissionType());
                permissionBO.setPermissionUrl(permissionRoleDTO.getPermissionUrl());
                permissionQuery.add(permissionBO);
            }
        }
    }

    @Override
    public List<RoleBO> listRoleBO(String roleIds) throws SearchException {
        String[] roleIdArray = roleIds.split("_");
        List<RoleBO> roleBOs = new ArrayList<>();
        for (String roleId : roleIdArray) {
            RoleBO roleBO = getRoleBOByRoleId(roleId);
            roleBOs.add(roleBO);
        }
        return roleBOs;
    }

    @Override
    public SuccessResult updateRoleDataAuthority(String roleId, RoleDataAuthorityVO roleDataAuthorityVO) throws SaveException, RemoveException {
        LOG.debug("更新角色数据权限");
        Map<String, Object> params = getHashMap(2);
        params.put("roleId", roleId);
        params.put("roleDataAuthority", roleDataAuthorityVO.getDataAuthorityType());
        roleDao.updateRole(params);
        if (StringUtils.equals(roleDataAuthorityVO.getDataAuthorityType(), RoleDataAuthorityEnum.CUSTOM.getDataAuthorityType())) {
            params.remove("roleDataAuthority");
            LOG.debug("删除原有的自定义权限列表");
            roleDao.deleteRoleDataAuthority(params);
            LOG.debug("更新自定义权限列表");
            for (RoleUserAuthorityVO roleUserAuthorityVO : roleDataAuthorityVO.getRoleUserAuthorityList()) {
                params.put("departmentId", roleUserAuthorityVO.getDepartmentId());
                params.put("userId", roleUserAuthorityVO.getUserId());
                roleDao.saveRoleDataAuthority(params);
            }
        }
        return new SuccessResult();
    }

    @Override
    public void setRoleMenu(RoleBO roleBO) throws SearchException {
        RoleBO role = getRoleBOByRoleId(roleBO.getRoleId());
        roleBO.setApiSaveMenu(role.getApiSaveMenu());
        roleBO.setApiDeleteMenu(role.getApiDeleteMenu());
        roleBO.setApiUpdateMenu(role.getApiUpdateMenu());
        roleBO.setApiQueryMenu(role.getApiQueryMenu());
        roleBO.setResourceSaveMenu(role.getResourceSaveMenu());
        roleBO.setResourceDeleteMenu(role.getResourceDeleteMenu());
        roleBO.setResourceUpdateMenu(role.getResourceUpdateMenu());
        roleBO.setResourceQueryMenu(role.getResourceQueryMenu());
        roleBO.setRouteSaveMenu(role.getRouteSaveMenu());
        roleBO.setRouteUpdateMenu(role.getRouteUpdateMenu());
        roleBO.setRouteQueryMenu(role.getRouteQueryMenu());
        roleBO.setPermissionInsert(role.getPermissionInsert());
        roleBO.setPermissionDelete(role.getPermissionDelete());
        roleBO.setPermissionUpdate(role.getPermissionUpdate());
        roleBO.setPermissionQuery(role.getPermissionQuery());
    }

    @Override
    public void setRoleMenus(List<RoleBO> roleBOs) throws SearchException {
        for (RoleBO roleBO : roleBOs) {
            setRoleMenu(roleBO);
        }
    }

    @Override
    public RoleDataAuthorityTypeDTO getRoleDataAuthorityType(String roleId) throws SearchException {
        Map<String, Object> params = getHashMap(1);
        params.put("roleId", roleId);
        RoleDTO roleDTO = roleDao.getRole(params);
        StringBuilder roleDataAuthority = new StringBuilder();
        if (StringUtils.equals(roleDTO.getRoleDataAuthority(), RoleDataAuthorityEnum.CUSTOM.getDataAuthorityType())) {
            List<RoleDataAuthorityDTO> roleDataAuthorityDTOs = roleDao.listRoleDataAuthority(params);
            for (RoleDataAuthorityDTO roleDataAuthorityDTO : roleDataAuthorityDTOs) {
                if (roleDataAuthority.length() > 0) {
                    roleDataAuthority.append("_");
                }
                roleDataAuthority.append(roleDataAuthorityDTO.getUserId());
            }
        }
        RoleDataAuthorityTypeDTO roleDataAuthorityTypeDTO = new RoleDataAuthorityTypeDTO();
        roleDataAuthorityTypeDTO.setRoleDataAuthorityType(roleDTO.getRoleDataAuthority());
        roleDataAuthorityTypeDTO.setRoleDataAuthority(roleDataAuthority.toString());
        return roleDataAuthorityTypeDTO;
    }

    @Override
    public List<RoleDataAuthorityDTO> listRoleDataAuthorityByRoleIdList(List<String> roleIds) throws SearchException {
        Map<String, Object> params = getHashMap(1);
        params.put("roleIds", roleIds);
        return roleDao.listRoleDataAuthority(params);
    }

    @Override
    public List<String> listDataAuthorityUserIdByRole(List<RoleBO> roleBOs) {
        List<String> userIds = new ArrayList<>();
        List<String> roleIds = new ArrayList<>();
        for (RoleBO roleBO : roleBOs) {
            roleIds.add(roleBO.getRoleId());
        }
        List<RoleDataAuthorityDTO> roleDataAuthorityDTOs = listRoleDataAuthorityByRoleIdList(roleIds);
        for (RoleDataAuthorityDTO roleDataAuthorityDTO : roleDataAuthorityDTOs) {
            userIds.add(roleDataAuthorityDTO.getUserId());
        }
        return userIds;
    }

    @Override
    public List<String> listDataAuthorityUserIdByDepartment(List<DepartmentBO> departmentBOs) {
        List<String> userIds = new ArrayList<>();
        if (departmentBOs.isEmpty()) {
            return userIds;
        }
        List<String> departmentIds = new ArrayList<>();
        for (DepartmentBO departmentBO : departmentBOs) {
            departmentIds.add(departmentBO.getDepartmentId());
        }
        return departmentUserService.listUserIdByDepartmentIds(departmentIds);
    }

    @Override
    public String getRoleDataAuthority(List<RoleBO> roleBOs) {
        String roleDataAuthority = RoleDataAuthorityEnum.SELF.getDataAuthorityType();
        for (RoleBO roleBO : roleBOs) {
            if (StringUtils.equals(RoleDataAuthorityEnum.ALL.getDataAuthorityType(), roleBO.getRoleDataAuthority())) {
                roleDataAuthority = RoleDataAuthorityEnum.ALL.getDataAuthorityType();
                break;
            }
            if (StringUtils.equals(RoleDataAuthorityEnum.DEPARTMENT.getDataAuthorityType(), roleBO.getRoleDataAuthority())) {
                roleDataAuthority = RoleDataAuthorityEnum.DEPARTMENT.getDataAuthorityType();
                break;
            }
            if (StringUtils.equals(RoleDataAuthorityEnum.CUSTOM.getDataAuthorityType(), roleBO.getRoleDataAuthority())) {
                roleDataAuthority = RoleDataAuthorityEnum.CUSTOM.getDataAuthorityType();
                break;
            }
        }
        return roleDataAuthority;
    }

    @Override
    public void setGroupRoles(List<GroupBO> groupRoles, List<RoleBO> roleBOs) {
        Map<String, Object> params = getHashMap(1);
        groupRoles.forEach(groupBO -> {
            params.put("groupId", groupBO.getGroupId());
            roleBOs.addAll(listRoleBOByGroup(params));
        });
    }

    @Override
    public void removalDuplicateRole(List<RoleBO> roleBOs) {
        for (int i = 0; i < roleBOs.size(); i++) {
            RoleBO roleBO = roleBOs.get(i);
            boolean isExist = false;
            for (int j = i + 1; j < roleBOs.size(); j++) {
                if (StringUtils.equals(roleBO.getRoleId(), roleBOs.get(j).getRoleId())) {
                    isExist = true;
                    break;
                }
            }
            if (isExist) {
                roleBOs.remove(i);
                i--;
                break;
            }
        }
    }

    @Override
    public Integer countRole(Map<String, Object> params) throws SearchException {
        Integer roleCount = roleDao.countRole(params);
        return roleCount == null ? 0 : roleCount;
    }

    /**
     * 递归查询子角色
     *
     * @param roleDTOs
     * @param params
     * @throws SearchException
     */
    private void listSubRoles(List<RoleDTO> roleDTOs, Map<String, Object> params) throws SearchException {
        for (RoleDTO roleDTO : roleDTOs) {
            params.put("roleParentId", roleDTO.getRoleId());
            List<RoleDTO> subRoleDTOs = roleDao.listRole(params);
            roleDTO.setSubRoles(subRoleDTOs);
            listSubRoles(subRoleDTOs, params);
        }
    }

    /**
     * 获取code
     *
     * @param parentCode
     * @param parentId
     * @return
     * @throws SearchException
     */
    private String getCode(String parentCode, String parentId) throws SearchException {
        RoleDTO roleDTO = roleDao.getLastByParentId(parentId);
        String code = roleDTO != null ? roleDTO.getRoleCode() : "0000";
        return super.getNewCode(code, parentCode);
    }

}
